@@ -25,18 +25,22 @@ from trezorlib.transport import get_transport
 
 MULTISEND_CALL_ONLY = '0x40A2aCCbd92BCA938b02010E17A5b8929b49130D'
 multisends = {
+    10: '0x998739BFdAAdde7C933B942a68053933098f9EDa',
     250: '0x10B62CC1E8D9a9f1Ad05BCC491A7984697c19f7E',
+    288: '0x2Bd65cd56cAAC777f87d7808d13DEAF88e54E0eA',
     43114: '0x998739BFdAAdde7C933B942a68053933098f9EDa'
 }
 transaction_service = {
     1: 'https://safe-transaction.mainnet.gnosis.io',
     4: 'https://safe-transaction.rinkeby.gnosis.io',
     5: 'https://safe-transaction.goerli.gnosis.io',
+    10: 'https://safe-transaction.optimism.gnosis.io',
     56: 'https://safe-transaction.bsc.gnosis.io',
     100: 'https://safe-transaction.xdai.gnosis.io',
     137: 'https://safe-transaction.polygon.gnosis.io',
-    250: 'https://safe.fantom.network',
+    250: 'https://safe-txservice.fantom.network',
     246: 'https://safe-transaction.ewc.gnosis.io',
+    288: 'https://safe-transaction.mainnet.boba.network',
     42161: 'https://safe-transaction.arbitrum.gnosis.io',
     43114: 'https://safe-transaction.avalanche.gnosis.io',
     73799: 'https://safe-transaction.volta.gnosis.io',
@@ -80,8 +84,7 @@ class ApeSafe(Safe):
         """
         Instantiate a Brownie Contract owned by Safe account.
         """
-        if not web3.isChecksumAddress(address):
-            address = web3.ens.resolve(address)
+        address = to_checksum_address(address) if is_address(address) else web3.ens.resolve(address)
         return Contract(address, owner=self.account)
 
     def pending_nonce(self) -> int:
@@ -296,12 +299,7 @@ class ApeSafe(Safe):
         """
         return self.estimate_tx_gas(safe_tx.to, safe_tx.value, safe_tx.data, safe_tx.operation)
 
-    def preview(self, safe_tx: SafeTx, events=True, call_trace=False, reset=True):
-        """
-        Dry run a Safe transaction in a forked network environment.
-        """
-        if reset:
-            chain.reset()
+    def preview_tx(self, safe_tx: SafeTx, events=True, call_trace=False) -> TransactionReceipt:
         tx = copy(safe_tx)
         safe = Contract.from_abi('Gnosis Safe', self.address, self.get_contract().abi)
         # Replace pending nonce with the subsequent nonce, this could change the safe_tx_hash
@@ -325,12 +323,23 @@ class ApeSafe(Safe):
             receipt.info()
             receipt.call_trace(True)
             raise ExecutionFailure()
+
         if events:
             receipt.info()
         if call_trace:
             receipt.call_trace(True)
         return receipt
 
+    def preview(self, safe_tx: SafeTx, events=True, call_trace=False, reset=True, include_pending=False):
+        """
+        Dry run a Safe transaction in a forked network environment.
+        """
+        if reset:
+            chain.reset()
+        if include_pending:
+            self.preview_pending(events=events, call_trace=call_trace)
+        return self.preview_tx(safe_tx, events=events, call_trace=call_trace)
+
     def execute_transaction(self, safe_tx: SafeTx, signer=None) -> TransactionReceipt:
         """
         Execute a fully signed transaction likely retrieved from the pending_transactions method.
@@ -363,4 +372,28 @@ class ApeSafe(Safe):
         Dry run all pending transactions in a forked environment.
         """
         for safe_tx in self.pending_transactions:
-            self.preview(safe_tx, events=events, call_trace=call_trace, reset=False)
+            self.preview_tx(safe_tx, events=events, call_trace=call_trace)
+
+    @staticmethod
+    def get_safe_txhash_from_execution_tx(tx: Union[str,TransactionReceipt]) -> str:
+        """
+        Get safe txhash from execution tx.
+        """
+        if isinstance(tx, str):
+            tx = chain.get_transaction(tx)
+        return tx.events['ExecutionSuccess']['txHash']
+
+    @staticmethod
+    def get_safe_nonce_from_execution_tx(tx: Union[str,TransactionReceipt]) -> int:
+        """
+        Get safe nonce from execution tx.
+        """
+        safe_txhash = ApeSafe.get_safe_txhash_from_execution_tx(tx)
+        return ApeSafe.get_safe_nonce_from_safe_tx(safe_txhash)
+
+    @staticmethod
+    def get_safe_nonce_from_safe_tx(safe_txhash: str) -> int:
+        """
+        Get safe nonce from safe txhash.
+        """
+        return requests.get(f"{transaction_service[chain.id]}/api/v1/multisig-transactions/{safe_txhash}").json()['nonce']
